/******************************************************************/
/*                                                                */
/* PARTI : Parallel Automated Runtime Tools from ICASE            */
/* Version 2.0          30 June 1993                              */
/*                                                                */
/* This is the header file for the parti primitives.              */
/* This only includes stuff that a user might need to see.        */
/*								  */
/******************************************************************/

#if defined INTEL
#  include "machine/intel.h"
#elif defined PARAGON
#  include "machine/paragon.h"
#elif defined CM5
#  include "machine/cm5.h"
#elif defined SP1
#  include "machine/sp1.h"
#elif defined PVM3
#  include "machine/parti_pvm3.h"
#endif

/* include header files */
#include "op.h"


/*=============== SYSTEM INFORMATION========================== */
extern	int	PARTI_numnodes;	/* number of processors */
extern	int	PARTI_mynode;	/* my processor id */
extern	int	PARTI_pid;	/* process id */

/*================ LOGICAL VALUES =============================*/
#define TRUE		1
#define FALSE		0


/*================ SCHEDULE  STRUCTURE ========================*/
/* compiler communication pattern */
struct com_pattern {
  int n_msgs_to_recv;		/* number of messages going to receive */
  int n_msgs_to_send;		/* number of messages going to send */
  int dimen;			/* dimension of array */
  int *send_size;		/* total size of sending messages */
  int *fetch_size;		/* total size of receving messages */
  int total_send_size;		/* total size of sending messages */
  int total_fetch_size;		/* total size of receving messages */
  int buffer_size;		/* size of largest message */
  int *send_n,**send_list;	/* sizes and lists of sending messages */
  int *fetch_n,**fetch_list;	/* sizes and lists of receiving messages */
  int *gcolor;			/* coloring */
};

/* communication pattern (is a compiler pattern + perm vector to unpack) */
#define SCHED struct schedule_struct
struct schedule_struct {
  struct com_pattern *sr;
  int ndata;
  int *perm, *size;
  int *global_refs,*local_refs;
  int type;			/* non-contiguity check */
};

#define   NORMAL   0        /*   normal  schedule - original parti */
#define   NC       1        /*   for scattering from non-contiguous buffers */
#define   PROCONLY 2        /*   for scatter-appends, only proc info needed */
#define   VARSIZE  3        /*   variable size data transmitted */

/* Maximum dimensions to be set here when using block gather  */
#define MAXDIM  5        



/*===========   TRANSLATION  TABLE  ===============================*/
#include "ttable.h"


/*================= BUFFERS USED BY  PARTI ===========================*/

/* working buffer for parti for communcation purpose */
struct WBuffer {
	int	max_size;		/* size of working buffer */
	int	*msg_id;		/* message id for asynchronous comm */
	char	*buffer;		/* working buffer */
	char	**blist;		/* pointers of buffer */
	char	**wlist;		/* pointers of working buffer */
} ;
struct WBuffer *wbuffer;

/*============ HASH  TABLE  SUPPORT ==================================*/

#define CHAINSIZE             1 
#define PERCENT_DATA_RANGE    25 
#define HARDWIRED_HASH_SIZE   0

#define  MAX_STAMPS 32
#define  NO_MASK    -1
#define  NO_STAMP   -1
#define  FIRST_STAMP 0

typedef	struct rephcell {
  int	   offset;
  int	   stamp;
} REPHCELL;	/* hash cells for replicated translation table */

typedef struct hashcell  HASHCELL;

struct hashcell{
  int      tag ;
  int      key   [CHAINSIZE];
  int      proc  [CHAINSIZE];
  int      local [CHAINSIZE];
  int      stamp [CHAINSIZE];
  int      offset[CHAINSIZE];
  HASHCELL *next;
};

#define	   DEFHASH	0		/* default hash table */
#define	   REPHASH	1		/* replicated table */

typedef struct hashtable HASHSTR ;

struct  hashtable {
   int	    type;			/* type of hash cells */
   int      no_buckets ;
   int      first_offset;
   int      first_stamp ; 
   int      stamp_mask   ;
   int      no_stamps ; 
   int      start_offset[ MAX_STAMPS ];
   int      size_stamp[ MAX_STAMPS ];
   int      area_stamp[ MAX_STAMPS ];
   HASHCELL *buckets ;
   REPHCELL *cells;			/* cells for replicated table */
   TTABLE   *trans_table;		/* translation table */
};

/*=================== FUNCTION DECLARATIONS ========================*/

/* memory allocation functions */
#define MALLOC(x)      ((x *) malloc(sizeof(x)))
#define CALLOC(n, x)   ((x *) calloc(n, sizeof(x)))

/* for debug version */
#define malloc(n) debug_malloc(n)

/* function declarations */
struct com_pattern *schedule_gather();
void new_execute_gather();
void new_execute_scatter(), scatter_func();

/* user-level prototypes */
SCHED *schedule(),*ischedule();
SCHED *schedulen();
void dref();
void gather(),igather(),fgather(),dgather(),cgather();
void scatter(),iscatter(),fscatter(),dscatter(),cscatter();
void iscatter_add(),fscatter_add(),dscatter_add(),cscatter_add();
void iscatter_sub(),fscatter_sub(),dscatter_sub(),cscatter_sub();
void iscatter_mult(),fscatter_mult(),dscatter_mult(),cscatter_mult();
void PARTI_gather();
void PARTI_scatter();
void new_dref(),free_table();
void regdereference();

/* hash table functions */
HASHSTR *PARTI_create_hash_table(),*PARTI_create_rep_hash_table();
void  PARTI_free_hash_table();
void  PARTI_hash(),PARTI_schedule(),PARTI_schedule_nc();
void  PARTI_incremental_schedule(),PARTI_clear_mask(),PARTI_clear_stamp();

/* utils prototypes */
void check_point();
void print_com_pattern();
void bsort();
void check_sum();
/*================================================================*/
